package com.iflytek.cloud;

import android.content.Context;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.SpeechEvaluatorImpl;
import com.iflytek.msc.MSC;

/**
 * <h3>语音评测类</h3>
 * <p>
 * 		语音评测是通过智能语音技术自动对发音水平进行评价、发音错误、缺陷定位和问题分
 * 析的软件系统。评音评测提供汉语、英语两种语言的评测，支持单字（汉语专有）、词语
 * 和句子朗读三种题型，请参考{@link SpeechConstant#ISE_CATEGORY}。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createEvaluator}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #createEvaluator}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createEvaluator}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 * @see EvaluatorListener
 * @see EvaluatorResult
 */
public class SpeechEvaluator extends SpeechInterface{
	
	private static SpeechEvaluator sInstance = null;

	private SpeechEvaluatorImpl mEvaluator = null;
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getEvaluator}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 评测对象
	 * @see #destroy
	 * @see #getEvaluator
	 */
	public static SpeechEvaluator createEvaluator(Context context, InitListener listener) {
		synchronized( sSync ){
			if( null==sInstance && null!=SpeechUtility.getUtility() ) {
				sInstance = new SpeechEvaluator(context, null);
			}
		}//end of sync
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createEvaluator}创建单例对象。
	 * </p>
	 * 
	 * @return 评测对象
	 * 
	 * @see #createEvaluator
	 * @see #destroy
	 */
	public static SpeechEvaluator getEvaluator() {
		return sInstance;
	}
	
	protected SpeechEvaluator(Context context, InitListener listener) {
		super();
		
		mEvaluator = new SpeechEvaluatorImpl(context);
		
		if (null != listener) {
			// TODO 初始化回调处理 
		}
	}
	
	/**
	 * <h4>开始评测</h4>
	 * <p>
	 * 		调用此函数，开始评测。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始评测前，通过{@link #setParameter}设置相
	 * 应的参数。
	 * </p>
	 * <p>
	 * 		调用本函数开始评测后，通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），或由应用层调用
	 * {@link #writeAudio}写入音频流，获取用于评测的音频。在完成音频
	 * 录入（包括麦克风或写音频流方式）后，通过调用{@link #stopEvaluating}告知SDK已完
	 * 成音频录入，或由SDK自带的VAD（Voice Activity Detection,静音抑制）自动结束音频录
	 * 入，见{@link EvaluatorListener#onEndOfSpeech}。（关于写入音流方式的SDK自动结束
	 * 录入应注意的地方，请参考{@link #writeAudio}；VAD端点长度设置，请
	 * 参考{@link #setParameter}）。
	 * </p>
	 * <p>
	 * 		可通过{@link #cancel}取消当前的会话。
	 * </p>
	 * 
	 * @param text 评测文本
	 * @param textParams 当前会话参数，见{@link #setParameter}支持的参数，
	 * 		仅在当次会话生效，如无特殊要求，传入null即可。
	 * @param listener 监听器，获取状态和结果。
	 * 
	 * @return 
	 * 			  是否成功写入数据，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #writeAudio
	 * @see #createEvaluator
	 * @see #getEvaluator
	 * @see #stopEvaluating
	 * @see #setParameter
	 * @see #cancel
	 * @see #startEvaluating(byte[], String, EvaluatorListener)
	 */
	public int startEvaluating(String text, String textParams, EvaluatorListener listener) {
		if(mEvaluator == null) {
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		mEvaluator.setParameter(mSessionParams);
		return mEvaluator.startEvaluating(text, textParams, listener);
	}

	/**
	 * <h4>开始评测</h4>
	 * <p>
	 * 		传入byte[]类型的评测文本，开始评测会话。其他介绍请参考{@link #startEvaluating}。
	 * </p>
	 * 
	 * @see #startEvaluating(String, String, EvaluatorListener)
	 */
	public int startEvaluating(byte[] text, String textParams, EvaluatorListener listener) {
		if(mEvaluator == null) {
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		mEvaluator.setParameter(mSessionParams);
		return mEvaluator.startEvaluating(text, textParams, listener);
	}
	
	/**
	 * <h4>写入录音数据</h4>
	 * <p>
	 * 		通过调用此函数，把音频数据传给SDK。
	 * </p>
	 * <p>
	 * 		仅在{@link SpeechConstant#AUDIO_SOURCE}值为-1时，需要通过调用本函数，把
	 * 音频数据传给SDK。在写入音频数据前，应先通过{@link #startEvaluating}
	 * 开启会话。在结束录音数据写入后，调用{@link #stopEvaluating}告知应用层，当前会话
	 * 音频数据已全部写入。
	 * </p>
	 * <p>
	 * 		当SDK回调{@link EvaluatorListener#onEndOfSpeech}时，说明SDK已经通过VAD检测
	 * 到了静音末端点，应该立即停止写入音频数据。当SDK回调
	 * {@link EvaluatorListener#onResult}返回完结果，或回调
	 * {@link EvaluatorListener#onError}返回错误时，也应该停止当次会话的音频写
	 * 入。
	 * </p>
	 * <p>
	 * 		支持的音频格式，请参考{@link #setParameter(String, String)}的参数说明。
	 * </p>
	 * 
	 * @param buffer
	 *            要写入的录音数据缓存
	 * @param offset
	 *            实际音频在传入缓存的起始点
	 * @param length
	 *            音频数据长度
	 * @return 
	 * 			  是否成功写入数据，true 表示成功，false 表示有错误。发生错误时，通过
	 * 		{@link EvaluatorListener#onError}获取错误信息。
	 * 
	 * @see #startEvaluating
	 * @see #stopEvaluating
	 * @see EvaluatorListener
	 */
	public boolean writeAudio(byte[] buffer, int offset, int length) {
		//TODO 需要测试写音频是否可用
		if(null != mEvaluator && mEvaluator.isEvaluating())
			return mEvaluator.writeAudio(buffer, offset, length);
		else {
			DebugLog.LogE("SpeechEvaluator writeAudio failed, is not running");
			return false;
		}
	}

	/**
	 * <h4>是否在会话中</h4>
	 * <p>
	 * 		通过此函数，获取当前SDK是否正在进行会话。应用层可通过此函数，查询能否
	 * 开始一路新的会话等。
	 * </p>
	 * <p>
	 * 		调用了{@link #stopEvaluating}停止录音后，如果会话未出现错误或返回最后的结果，
	 * 当前状态依然处于会话中，即本函数会返回true。如果调用{@link #cancel}取消了会话，
	 * 则当前状态处于不在会话中。
	 * </p>
	 * @return 会话状态，true：正在会话中；false：不在会话中。
	 * 
	 * @see #startEvaluating
	 * @see #stopEvaluating
	 * @see #cancel
	 */
	public boolean isEvaluating() {
		if(null != mEvaluator && mEvaluator.isEvaluating()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * <h4>停止录音</h4>
	 * <p>
	 * 		调用本函数告知SDK，当前会话音频已全部录入。
	 * </p>
	 * <p>
	 * 		在调用本函数后，已录入的音频还在继续上传到服务器，结果不会马上就返回，当前会
	 * 话还在继续，直到结果返回完毕，或出现错误。
	 * </p>
	 * <p>
	 * 		要取消会话，请参考{@link #cancel}函数。
	 * </p>
	 * <p>
	 * 		当应用调用本函数结束停止录音时，SDK不会再回调
	 * {@link EvaluatorListener#onEndOfSpeech}。相反的，当SDK回调
	 * {@link EvaluatorListener#onEndOfSpeech}时，应用层可不必再调用本函数通知SDK停止
	 * 录音。
	 * </p>
	 * 
	 * @see #startEvaluating
	 * @see #writeAudio
	 * @see EvaluatorListener#onEndOfSpeech
	 * @see #cancel
	 */
	public void stopEvaluating() {
		if(null != mEvaluator && mEvaluator.isEvaluating()) {
			mEvaluator.stopEvaluating();
		} else {
			DebugLog.LogE("SpeechEvaluator stopEvaluating failed, is not running");
		}
	}

	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		通过此函数取消当前的会话。
	 * </p>
	 * <p>
	 * 		在会话被取消后，当前会话结束，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startEvaluating
	 * @see #stopEvaluating
	 */
	public void cancel() {
		if(null != mEvaluator && mEvaluator.isEvaluating()) {
			mEvaluator.cancel(false);
		}
	}
	
	/**
	 * <h4>设置参数</h4>
	 * <p>
	 * 		设置评测会话参数。
	 * </p>
	 * <p>
	 * 		可设置参数有：<br>
	 * {@link SpeechConstant#ISE_AUDIO_PATH} 音频保存路径<br>
	 * {@link SpeechConstant#ISE_AUTO_TRACKING} 朗读跟踪<br>
	 * {@link SpeechConstant#ISE_CATEGORY} 题型<br>
	 * {@link SpeechConstant#LANGUAGE} 语言<br>
	 * {@link SpeechConstant#ISE_INTERRUPT_ERROR} 中断报错<br>
	 * {@link SpeechConstant#ISE_PARSED} 是否已标注<br>
	 * {@link SpeechConstant#ISE_SOURCE_PATH} 音频源文件路径<br>
	 * {@link SpeechConstant#ISE_TRACK_TYPE} 跟踪模式<br>
	 * {@link SpeechConstant#ISE_USER_MODEL_ID} 用户模型ID<br>
	 * {@link SpeechConstant#KEY_REQUEST_FOCUS} 音频焦点<br>
	 * {@link SpeechConstant#NET_TIMEOUT} 网络超时<br>
	 * {@link SpeechConstant#RESULT_LEVEL} 结果等级<br>
	 * {@link SpeechConstant#SAMPLE_RATE} 音频采样率<br>
	 * {@link SpeechConstant#TEXT_BOM} 文本字节顺序<br>
	 * {@link SpeechConstant#TEXT_ENCODING} 文本编码格式<br>
	 * {@link SpeechConstant#VAD_ENABLE} 启用VAD<br>
	 * {@link SpeechConstant#VAD_BOS} VAD前端点<br>
	 * {@link SpeechConstant#VAD_EOS} VAD后端点
	 * <br>{@link SpeechConstant#AUDIO_SOURCE}: 音频源
	 * <br>{@link SpeechConstant#BLUETOOTH}: 蓝牙录音
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型（目前评测仅支持在线模式）；
	 * </p>
	 * <p>
	 * 		当前评测支持未压缩的16位，单声道，采样率为16000或8000，字节
	 *  顺序为Little-Endian的Windows PCM音频。
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return true 设置成功，false 设置失败
	 */
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}

	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#SAMPLE_RATE}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createEvaluator}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isEvaluating}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createEvaluator}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createEvaluator
	 * @see #cancel
	 */
	public boolean destroy() {
		boolean ret = true;
		
		final SpeechEvaluatorImpl impl = mEvaluator;
		if( null != impl ){
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
		}
		
		return ret;
	}
}
