/*
 * 文件名：SpeechRecognizer.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：识别对象，包括开始识别、取消识别、停止识别等接口；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.RemoteException;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.param.MscKeys;
import com.iflytek.cloud.speech.impl.SpeechRecognizerImpl;
import com.iflytek.cloud.util.ResourceUtil;
import com.iflytek.speech.SpeechRecognizerAidl;

/**
 * <h3>语音识别类</h3>
 * <p>
 * 		语音识别，包括听写、语法识别功能。语音识别技术(Auto Speech Recognize，
 * 简称ASR)即把人的自然语言音频数据转换成文本数据。除了本类包含的听写、语法
 * 识别外，还有语义理解类{@link SpeechUnderstander}。关于文本数据转语音的
 * 功能，请参考语音合成类 {@link SpeechSynthesizer}。关于识别对话框，请参考
 * {@link com.iflytek.cloud.ui.RecognizerDialog}。
 * </p>
 * <p>
 * 		听写，是基于自然语言处理，将自然语言音频转换为文本输出的技术。语音听写技术
 * 与语法识别技术的不同在于，语音听写不需要基于某个具体的语法文件，其识别
 * 范围是整个语种内的词条。在听写时，应用还可以上传个性化的词表，如联系人
 * 列表等，提高列表中词语的匹配率。关于个性化词表上传，请参考{@link
 * #updateLexicon(String, String, LexiconListener)}。
 * </p>
 * <p>
 * 		语法识别，是基于语法规则，将与语法一致的自然语言转换为文本输出的技术。语法
 * 识别相比听写，有更高的匹配率，多用于要更准确结果且有限说法的语音控制，
 * 如空调的语音控制等。在使用语法识别时，应用需要先构建一个语法文件上传给
 * 服务器，并在会话时，传入语法ID，以使用该语法。当在线语法识别使用网页生成语法时，
 * 不需要传入语法ID——即不需要再在应用中上传语法文件（除非应用需要使用另外的语法文件），
 * 这时，设置{@link SpeechConstant#SUBJECT}参数值为{@link SpeechConstant#ENG_ASR}即可。
 * </p>
 * <p>
 *       关于上传语法文件，请参考 {@link #buildGrammar(String, String, GrammarListener)}。
 * <script>gShowGrammarGuide();</script>。
 * 关于语法ID设置，请参考{@link #setParameter(String, String)}和
 * {@link SpeechConstant#CLOUD_GRAMMAR}、
 * {@link SpeechConstant#LOCAL_GRAMMAR}。
 * </p>
 * <p>
 * 		听写与语法识别的区别，在于语法识别需要指写一个语法ID，而听写不需要。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createRecognizer}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getRecognizer()}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createRecognizer}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 * @see SpeechUnderstander
 * @see SpeechSynthesizer
 * @see com.iflytek.cloud.ui.RecognizerDialog
 */
public final class SpeechRecognizer extends SpeechInterface{
		
	/**
	 *  识别单例对象
	 */
	private static SpeechRecognizer sInstance = null;
	
	/**
	 *  客户端插件MSC识别实例
	 */
	private SpeechRecognizerImpl mRecognizer = null;	
	
	/**
	 *  语音+识别实例
	 */
	private com.iflytek.speech.SpeechRecognizerAidl mRecognizerAidl = null;

	/**
	 *  语音+识别监听
	 */
	private InternalListener mInternalAidlListener = null;
	
	/**
	 * 初始化回调监听
	 */
	private InitListener mInitListener = null;
	
	
	/**
	 * 内部识别监听类，保证回调在主线程完成
	 * 
	 *
	 */
	private final class InternalListener implements RecognizerListener{
		private RecognizerListener mOutListener = null;
		private com.iflytek.speech.RecognizerListener mAidlListener = null;
		
		
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		// EVENT自定义消息
		private static final int MSG_EVENT  = 6;
		
		public InternalListener(RecognizerListener outListener) {
			mOutListener = outListener;
			mAidlListener = new com.iflytek.speech.RecognizerListener.Stub() {

				@Override
				public void onVolumeChanged(int volume, byte[] data) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onError(int errorCode) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_ERROR, new SpeechError(errorCode));
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onEndOfSpeech() throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END);
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onBeginOfSpeech() throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN);
					mUiHandler.sendMessage(msg);
				}

				@Override
				public void onResult(
						com.iflytek.speech.RecognizerResult result,
						boolean islast) throws RemoteException {
					
					Message msg = mUiHandler.obtainMessage(MSG_RESULT, 
							islast == true ? 1 : 0, 0, new RecognizerResult(result.getResultString()));
					mUiHandler.sendMessage(msg);
				}

				@Override
				public void onEvent(int eventType, int arg1, int arg2,
						Bundle obj) throws RemoteException {
					Message msgbody = new Message();
					msgbody.what = eventType;
					msgbody.arg1 = arg1;
					msgbody.arg2 = arg2;
					msgbody.obj  = obj;
					Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0,0, msgbody);
					mUiHandler.sendMessage(tmsg);
					
				}
			};
		}
		
		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_AUDIO_BUFFER:
					mOutListener.onVolumeChanged(msg.arg1,(byte[]) msg.obj);
					break;
				case MSG_SPEECH_BEGIN:
					mOutListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					mOutListener.onEndOfSpeech();
					break;
				case MSG_RESULT:
				{
					mOutListener.onResult((RecognizerResult) msg.obj,
											msg.arg1==1?true:false);
				}
					break;
				case MSG_EVENT:
					Message mbody = (Message)msg.obj;
					// 自定义消息数据
					mOutListener.onEvent(mbody.what,mbody.arg1,mbody.arg2,(Bundle)mbody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};
		
		@Override
		public void onVolumeChanged(int volume, byte[] data) {
			Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEndOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onResult(RecognizerResult result, boolean isLast) {
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, 
								isLast == true ? 1 : 0, 0, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			
			Message msgbody = new Message();
			msgbody.what = eventType;
			msgbody.arg1 = arg1;
			msgbody.arg2 = arg2;
			msgbody.obj  = obj;
			
			Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0,0, msgbody);
			mUiHandler.sendMessage(tmsg);
		}
		
	}
	
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getRecognizer()}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 识别对象
	 * @see #destroy
	 * @see #getRecognizer
	 */
	public synchronized static SpeechRecognizer createRecognizer(Context context,InitListener listener) {
		synchronized( sSync ){
			if( sInstance == null && null!=SpeechUtility.getUtility() ){
				sInstance = new SpeechRecognizer(context,listener);
			}
		}//end of sync
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createRecognizer}创建单例对象。
	 * </p>
	 * 
	 * @return 识别对象
	 * 
	 * @see #createRecognizer
	 * @see #destroy
	 */
	public static SpeechRecognizer getRecognizer() {
		return sInstance;
	}
	
	/**
	 * <h4>构造函数</h4>
	 * <p>
	 * 		本类使用单例，应用层不需要直接调用构造函数，所以构造函数属性为非public。
	 * </p>
	 * @see #createRecognizer
	 * @see #destroy
	 * @see #getRecognizer()
	 */
	protected SpeechRecognizer(Context context,final InitListener listener) {
		super();
		
		mInitListener = listener;
		
		// 分别创建speech.SpeechRecognizerImpl及com.iflytek.speech.SpeechRecognizerAidl
		mRecognizer = new SpeechRecognizerImpl(context);
		
		//创建语音+识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
			&& utility.getEngineMode() != ENGINE_MODE.MSC) {
			mRecognizerAidl = new SpeechRecognizerAidl(context.getApplicationContext(), listener);
		} else if(null != listener) {
			//放在主线程中回调，修复用户在回调中立即使用本对象造成的空指针问题
			Message.obtain(mUiHandler, ErrorCode.SUCCESS, 0, 0, null).sendToTarget();
		}
	}
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
		@Override
		public void handleMessage(Message msg) {
			//listener 判断是否为null
			if(mInitListener == null)
				return;
			mInitListener.onInit(ErrorCode.SUCCESS);
		}
	};
	
	protected void reInit(Context context) 
	{	
		//创建语音+识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
				&& utility.getEngineMode() != ENGINE_MODE.MSC) {
			//判断服务是否可用，在用户升级语音+造成的服务端开时会出现服务不可用的情况
			if(null != mRecognizerAidl && !mRecognizerAidl.isAvailable()){
				mRecognizerAidl.destory();
				mRecognizerAidl = null;
			}
			mRecognizerAidl = new SpeechRecognizerAidl(context.getApplicationContext(), mInitListener);
		} else if(null != mInitListener) {
			if(null != mRecognizerAidl){
				mRecognizerAidl.destory();
				mRecognizerAidl = null;
			}
		}
	}
	
	/**
	 * <h4>构建语法文件</h4>
	 * <p>
	 * 		在使用语法识别时，需要先构建语法。在线语法识别，并已通过网页发布语法的，可不
	 * 通过此函数构建语法。参考本类开头的介绍。
	 * </p>
	 * <p>
	 * 		本地语法仅支持"bnf"类型，构建在线语法支持“abnf”。
	 * </p>
	 * <p>
	 * 		<script>gShowGrammarGuide();</script>。
	 * </p>
	 * 
	 * @param grammarType
	 *            语法类型，支持：abnf，bnf
	 * @param grammarContent
	 *            语法内容
	 * @param grammarListener
	 *            语法构建回调，语法ID将通过
	 *            {@link GrammarListener#onBuildFinish(String, SpeechError)}回调返回，以用于
	 *            识别开始前设置{@link SpeechConstant#CLOUD_GRAMMAR}或
	 *            {@link SpeechConstant#LOCAL_GRAMMAR}参数，使用该语法。
	 */
	public int buildGrammar(String grammarType, String grammarContent,
			GrammarListener grammarListener)
	{
		//语音+已经安装，并且本地或者混合识别,需要走语音+
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_ASR, mRecognizerAidl);
		DebugLog.LogD("start engine mode = " + mode.toString());

		// @{ start: 去除语记支持
		//本地构建,需要走语音+
		/*if(mode == ENGINE_MODE.PLUS)
		{
			//没有安装语音+
			if(mRecognizerAidl == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, null);
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			
			//TODO 此处需要测试非主线程调用时候是否能够在主线程抛出回调消息
			final GrammarListener finalListener = grammarListener; 
			com.iflytek.speech.GrammarListener aidlGrammarListener = new com.iflytek.speech.GrammarListener.Stub() {
				@Override
				public void onBuildFinish(String grammarId, int errorCode)
						throws RemoteException {
					if(finalListener != null) {
						finalListener.onBuildFinish(grammarId, ErrorCode.SUCCESS == errorCode ? null : new SpeechError(errorCode));
					}
				}
			};
			return mRecognizerAidl.buildGrammar(grammarType, grammarContent, aidlGrammarListener);
		} else {
			if(mRecognizer == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mRecognizer.setParameter(mSessionParams);
			return mRecognizer.buildGrammar(grammarType, grammarContent, grammarListener);
		}*/
		// @ end:去除语记支持}
		if(mRecognizer == null)
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		mRecognizer.setParameter(mSessionParams);
		return mRecognizer.buildGrammar(grammarType, grammarContent, grammarListener);
	}

	/**
	 * <h4>更新词典</h4>
	 * <p>
	 * 		更新应用个性化的词典，如联系人列表等。热词更新后，在5~10分钟内生效。
	 * </p>
	 * 
	 * @param lexiconName
	 *            词典名称，{ "userword", "contact" }。
	 * @param lexiconContent
	 *            词典内容，联系人列表：以换行符"\n"分隔的词语字符串，如"张三，李四，王五";
	 *            用户热词：满足一定格式的json字符串，格式如下：
	 * <br>{
	 * <br>&nbsp;"userword":
	 *		<br>&nbsp;&nbsp;[
	 *			<br>&nbsp;&nbsp;&nbsp;{ "name" : "default" , "words" : [ "默认词条1", "默认词条2" ] },
	 *			<br>&nbsp;&nbsp;&nbsp;{ "name" : "词表名称1", "words": [ "词条1的第一个词", "词条1的第二个词"] },
	 *	        <br>&nbsp;&nbsp;&nbsp;{ "name" : "词表名称2", "words": [ "词条2的第一个词", "词条2的第二个词"] }
	 *		<br>&nbsp;&nbsp;]
	 * <br>}
	 * <br>同时请参考相关DEMO的示例, 或{@link com.iflytek.cloud.util.UserWords}类。
	 * 
	 * @param lexiconListener
	 *            词典更新回调接口
	 */
	public int updateLexicon(String lexiconName, String lexiconContent,
			LexiconListener lexiconListener)
	{	
		//语音+已经安装，并且本地或者混合识别,需要走语音+
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_ASR, mRecognizerAidl);
		DebugLog.LogD("start engine mode = " + mode.toString());

		//@{start:去除语记支持
		//本地构建,需要走语音+
		/*if(mode == ENGINE_MODE.PLUS)
		//本地构建,需要走语音+
		{
			//没有安装语音+
			if(mRecognizerAidl == null)
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, null);
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			
			//TODO 此处需要测试非主线程调用时候是否能够在主线程抛出回调消息
			final LexiconListener finalListener = lexiconListener;
			com.iflytek.speech.LexiconListener aidlLexiconListener = new com.iflytek.speech.LexiconListener.Stub() {
				
				@Override
				public void onLexiconUpdated(String lexiconId, int errorCode)
						throws RemoteException {
					if (finalListener != null) {
						finalListener.onLexiconUpdated(lexiconId, ErrorCode.SUCCESS == errorCode ? null : new SpeechError(errorCode));
					}
				}
			};
			return mRecognizerAidl.updateLexicon(lexiconName, lexiconContent, aidlLexiconListener);
		} else {
			if(mRecognizer == null) 
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mRecognizer.setParameter(mSessionParams);
			return mRecognizer.updateLexicon(lexiconName, lexiconContent, lexiconListener);
		}*/
		//@ end:去除语记支持}

		if(mRecognizer == null)
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		mRecognizer.setParameter(mSessionParams);
		return mRecognizer.updateLexicon(lexiconName, lexiconContent, lexiconListener);
	}
	


	/**
	 * <h4>开始录音</h4>
	 * <p>
	 * 		调用此函数，开始听写或语法识别。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始听写或语法识别前，通过{@link #setParameter(String, String)}设置相
	 * 应的参数。
	 * </p>
	 * <p>
	 * 		调用本函数开始识别后，通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），或由应用层调用
	 * {@link #writeAudio(byte[], int, int)}写入音频流，获取用于识别的音频。在完成音频
	 * 录入（包括麦克风或写音频流方式）后，通过调用{@link #stopListening()}告知SDK已完
	 * 成音频录入，或由SDK自带的VAD（Voice Activity Detection,静音抑制）自动结束音频录
	 * 入，见{@link RecognizerListener#onEndOfSpeech()}。（关于写入音流方式的SDK自动结束
	 * 录入应注意的地方，请参考{@link #writeAudio(byte[], int, int)}；VAD端点长度设置，请
	 * 参考{@link #setParameter(String, String)}）
	 * </p>
	 * <p>
	 * 		可通过{@link #cancel()}取消当前的会话。
	 * </p>
	 * 
	 * @see #createRecognizer
	 * @see #getRecognizer()
	 * @see #stopListening()
	 * @see #setParameter(String, String)
	 * @see #cancel()
	 */
	public int startListening(RecognizerListener listener)
	{
		//语音+已经安装，并且本地或者混合识别,需要走语音+
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_ASR, mRecognizerAidl);
		DebugLog.LogD("start engine mode = " + mode.toString());

		//@{start:去除语记
		/*if(mode == ENGINE_MODE.PLUS) {
			//没有安装语音+
			if(mRecognizerAidl == null) 
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, null);
			mRecognizerAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			
			//主线程抛出回调消息
			mInternalAidlListener = new InternalListener(listener);				
			return mRecognizerAidl.startListening(mInternalAidlListener.mAidlListener);
		} else {
			if(mRecognizer == null) 
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mRecognizer.setParameter(mSessionParams);
			return mRecognizer.startListening(listener);
		}*/
		//@ end:去除语记}
		if(mRecognizer == null)
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		mRecognizer.setParameter(mSessionParams);
		return mRecognizer.startListening(listener);
	}
	
	
	
	
	/**
	 * <h4>写入录音数据</h4>
	 * <p>
	 * 		通过调用此函数，把音频数据传给SDK。
	 * </p>
	 * <p>
	 * 		仅在{@link SpeechConstant#AUDIO_SOURCE}值为-1时，需要通过调用本函数，把
	 * 音频数据传给SDK。在写入音频数据前，应先通过{@link #startListening(RecognizerListener)}
	 * 开启会话。在结束录音数据写入后，调用{@link #stopListening()}告知应用层，当前会话
	 * 音频数据已全部写入。
	 * </p>
	 * <p>
	 * 		当SDK回调{@link RecognizerListener#onEndOfSpeech()}时，说明SDK已经通过VAD检测
	 * 到了静音末端点，应该立即停止写入音频数据。当SDK回调
	 * {@link RecognizerListener#onResult}返回完结果，或回调
	 * {@link RecognizerListener#onError(SpeechError)}返回错误时，也应该停止当次会话的音频写
	 * 入。
	 * </p>
	 * <p>
	 * 		支持的音频格式，请参考{@link #setParameter(String, String)}的参数说明。
	 * </p>
	 * 
	 * @param buffer
	 *            要写入的录音数据缓存
	 * @param offset
	 *            实际音频在传入缓存的起始点
	 * @param length
	 *            音频数据长度
	 * @return 
	 * 			  是否成功写入数据，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #startListening(RecognizerListener)
	 * @see #stopListening()
	 * @see RecognizerListener
	 */
	public int writeAudio(byte[] buffer,int offset,int length)
	{
		//TODO 需要测试写音频是否可用
		if(null != mRecognizer && mRecognizer.isListening())
			return mRecognizer.writeAudio(buffer, offset, length);
		else if(null != mRecognizerAidl && mRecognizerAidl.isListening())
			return mRecognizerAidl.writeAudio(buffer, offset, length);
		else {
			DebugLog.LogE("SpeechRecognizer writeAudio failed, is not running");
			return ErrorCode.ERROR_ENGINE_CALL_FAIL;
		}
	}
	
	/**
	 * <h4>停止录音</h4>
	 * <p>
	 * 		调用本函数告知SDK，当前会话音频已全部录入。
	 * </p>
	 * <p>
	 * 		在调用本函数后，已录入的音频还在继续上传到服务器，结果不会马上就返回，当前会
	 * 话还在继续，直到结果返回完毕，或出现错误。
	 * </p>
	 * <p>
	 * 		要取消会话，请参考{@link #cancel()}函数。
	 * </p>
	 * <p>
	 * 		当应用调用本函数结束停止录音时，SDK不会再回调
	 * {@link RecognizerListener#onEndOfSpeech()}。相反的，当SDK回调
	 * {@link RecognizerListener#onEndOfSpeech()}时，应用层可不必再调用本函数通知SDK停止
	 * 录音。
	 * </p>
	 * 
	 * @see #startListening(RecognizerListener)
	 * @see #writeAudio(byte[], int, int)
	 * @see RecognizerListener#onEndOfSpeech()
	 * @see #cancel()
	 */
	public void stopListening()
	{
		if(null != mRecognizer && mRecognizer.isListening()) {
			mRecognizer.stopListening();
		} else if(null != mRecognizerAidl && mRecognizerAidl.isListening()) {
			if(mInternalAidlListener != null)
			{
				mRecognizerAidl.stopListening(mInternalAidlListener.mAidlListener);
			}
		} else {
			DebugLog.LogE("SpeechRecognizer stopListening failed, is not running");
		}
	}
	
	/**
	 * <h4>是否在会话中</h4>
	 * <p>
	 * 		通过此函数，获取当前SDK是否正在进行会话。应用层可通过此函数，查询能否
	 * 开始一路新的会话等。
	 * </p>
	 * <p>
	 * 		调用了{@link #stopListening()}停止录音后，如果会话未出现错误或返回最后的结果，
	 * 当前状态依然处于会话中，即本函数会返回true。如果调用{@link #cancel()}取消了会话，
	 * 则当前状态处于不在会话中。
	 * </p>
	 * @return 会话状态，true：正在会话中；false：不在会话中。
	 * 
	 * @see #startListening(RecognizerListener)
	 * @see #stopListening()
	 * @see #cancel()
	 */
	public boolean isListening()
	{
		if(null != mRecognizer && mRecognizer.isListening()) {
			return true;
		} else if(null != mRecognizerAidl && mRecognizerAidl.isListening()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		通过此函数取消当前的会话。
	 * </p>
	 * <p>
	 * 		在会话被取消后，当前会话结束，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startListening(RecognizerListener)
	 * @see #stopListening()
	 */
	public void cancel() {
		if(null != mRecognizer && mRecognizer.isListening()) {
			mRecognizer.cancel(false);
		} else if(null != mRecognizerAidl && mRecognizerAidl.isListening()) {
			if(mInternalAidlListener != null)
			{
				mRecognizerAidl.cancel(mInternalAidlListener.mAidlListener);
			}
		} else {
			DebugLog.LogE("SpeechRecognizer cancel failed, is not running");
		}
	}
	
	/**
	 * <h4>参数设置</h4>
	 * <p>
	 * 		可设置的参数有：<br>
	 * {@link SpeechConstant#NET_TIMEOUT }： 网络连接超时时间<br>
	 * {@link SpeechConstant#KEY_SPEECH_TIMEOUT }：语音输入超时时间 <br>
	 * {@link SpeechConstant#LANGUAGE }：语言<br>
	 * {@link SpeechConstant#ACCENT }：语言区域<br>
	 * {@link SpeechConstant#DOMAIN }：应用领域<br>
	 * {@link SpeechConstant#CLOUD_GRAMMAR }：云端语法ID<br>
	 * {@link SpeechConstant#LOCAL_GRAMMAR }：本地语法ID<br>
	 * {@link SpeechConstant#AUDIO_SOURCE }：音频源<br>
	 * {@link SpeechConstant#BLUETOOTH}: 蓝牙录音<br>
	 * {@link SpeechConstant#VAD_BOS }：前端点超时<br>
	 * {@link SpeechConstant#VAD_EOS }：后端点超时<br>
	 * {@link SpeechConstant#VAD_ENABLE }：启用VAD<br>
	 * {@link SpeechConstant#SAMPLE_RATE }：识别采样率<br>
	 * {@link SpeechConstant#ASR_NBEST }：句子级多候选<br>
	 * {@link SpeechConstant#ASR_WBEST }：词级多候选<br>
	 * {@link SpeechConstant#ASR_PTT }：设置是否有标点符号<br>
	 * {@link SpeechConstant#RESULT_TYPE }：识别结果类型<br>
	 * {@link SpeechConstant#ASR_AUDIO_PATH }：识别录音保存路径
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型；
	 * <br>{@link com.iflytek.cloud.util.ResourceUtil#ASR_RES_PATH}：离线资源路径；
	 * <br>{@link com.iflytek.cloud.util.ResourceUtil#ENGINE_START}：启动离线引擎；
	 * <br>{@link com.iflytek.cloud.util.ResourceUtil#GRM_BUILD_PATH}：离线语法路径；<br>
	 *     获取翻译结果，需增加如下参数:<br>
	 * {@link SpeechConstant#ASR_SCH }:语义或翻译<br>
	 * {@link SpeechConstant#ORI_LANG }: 原始语种<br>
	 * {@link SpeechConstant#TRANS_LANG }: 目标语种<br>
	 * {@link SpeechConstant#ADD_CAP }:语义或翻译处理通道<br>
	 * {@link SpeechConstant#TRS_SRC }:翻译结果格式<br>
	 * {@link SpeechConstant#DVC_INFO}: 设备信息<br>
	 * {@link SpeechConstant#AUDIO_FORMAT_AUE}:音频流编解码格式，非音频流操作并不支持此参数设置，如命令词识别
	 *
	 * </p>
	 * <p>
	 * 		当前识别支持未压缩的16位，单声道，采样率为16000或8000，字节顺序为
	 *  Little-Endian的Windows PCM音频。本地识别时，需要有相应的资源文件才支
	 *  持对应的采样率的音频识别。
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 是否设置成功
	 * 
	 * @see #getParameter(String)
	 * @see #startListening(RecognizerListener)
	 */
	@Override
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}
	
	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#SAMPLE_RATE}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createRecognizer}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isListening()}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createRecognizer}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createRecognizer
	 * @see #cancel()
	 */
	public boolean destroy() {
		boolean ret = true;
		final SpeechRecognizerAidl  aidl = mRecognizerAidl;
		if( null != aidl ) {
			aidl.destory();
		}
		
		synchronized( this ){
			mRecognizerAidl = null;
		}//end of sync
		
		final SpeechRecognizerImpl impl = mRecognizer;
		if( impl != null ){
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
			
			//销毁离线引擎和资源
			final SpeechUtility utility = SpeechUtility.getUtility();
			if( null != utility ){
				DebugLog.LogD( "Destory asr engine." );
				utility.setParameter( ResourceUtil.ENGINE_DESTROY, ResourceUtil.ENGINE_DESTROY+MscKeys.KEY_VAL_SEP+SpeechConstant.ENG_ASR );
			}
		}
		
		return ret;
	}
}
