/*
 * 文件名：SpeechUnderstander.java 2.5.1027.1040 Dec 11,2013
 * 描述 ：语义理解对象；
 * 作者:iFLytek
 * 版权：科大讯飞
 */
package com.iflytek.cloud;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.RemoteException;
import com.iflytek.cloud.SpeechConstant;
import com.iflytek.cloud.SpeechUnderstanderListener;
import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.SpeechUnderstanderImpl;
import com.iflytek.msc.MSC;
import com.iflytek.speech.SpeechUnderstanderAidl;

/**
 * <h3>语音语义类</h3>
 * <p>
 * 		语义理解，分语音语义和文本语义，主要是把自然语言内容，转换为有一定结构的文本数
 * 据，使应用能够抓取其中的重点数据，理解用户的使用意图，进行下一步的处理。语音语义，
 * 是语义理解的一种，其先把音频数据转为听写结果数据，再进行文本语义理解。关于文本语义，
 * 请参考{@link TextUnderstander}。关于语义理解结果说明，请参考
 * <script>gShowUnderstanderFile();</script>。当前语义理解仅有在线模式。
 * </p>
 * <p><script>gShowSchDefaultStatus();</script></p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createUnderstander}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getUnderstander()}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createUnderstander}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @since version:1073
 * @author <script>gShowAuthor();</script>
 * @see SpeechUtility#createUtility
 * @see SpeechRecognizer
 * @see SpeechSynthesizer
 */
public class SpeechUnderstander extends SpeechInterface{
	
	/**
	 * 语义场景。 
	 * 具体参数支持请参考开发文档(语义理解处理协议v1 3200.docx的1.3兴趣点描述相关规则)，示例：telephone|message|music|weather。
	 */
//	public static final String SCENE = "scene";

	/** 
	 * 语义理解单例对象
	 */
	protected static SpeechUnderstander sInstance = null;
	
	/**
	 * 客户端插件MSC语义理解实例
	 */
	private SpeechUnderstanderImpl  mUnderstander = null;
	
	/**
	 * 语记语义理解实例
	 */
	private SpeechUnderstanderAidl  mUnderstanderAidl = null;
	
	/**
	 *  语记识别监听
	 */
	private InternalListener mInternalAidlListener = null;
	
	/**
	 * 初始化回调监听
	 */
	private InitListener mInitListener = null;

	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getUnderstander()}获取已创建的单例对象。
	 * </p>
	 * 
	 * @return 语音语义对象
	 * @see #destroy
	 * @see #getUnderstander()
	 */
	public synchronized static SpeechUnderstander createUnderstander(Context context,InitListener listener) 
	{
		synchronized( sSync ){
			if ( sInstance==null && null!=SpeechUtility.getUtility() ){
				sInstance = new SpeechUnderstander(context,listener);
			}
		}//end of sync 
		
		return sInstance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createUnderstander}创建单例对象。
	 * </p>
	 * 
	 * @return 语音语义对象
	 * 
	 * @see #createUnderstander
	 * @see #destroy
	 */
	public static SpeechUnderstander getUnderstander() {
		return sInstance;
	}

	/**
	 * 初始化语义识别对象
	 * @param context 上下文环境
	 */
	protected SpeechUnderstander(Context context,InitListener listener) 
	{
		super();
		
		mInitListener = listener;
		
		// 分别创建speech.SpeechUnderstanderImpl及com.iflytek.speech.SpeechUnderstanderAidl
		mUnderstander = new SpeechUnderstanderImpl(context);
		
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
			&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			mUnderstanderAidl = new SpeechUnderstanderAidl(context.getApplicationContext(), listener);
		}
		else if(null != listener) 
		{
			//放在主线程中回调，修复用户在回调中立即使用本对象造成的空指针问题
			Message.obtain(mUiHandler, ErrorCode.SUCCESS, 0, 0, null).sendToTarget();
		}
	}
	
	private Handler mUiHandler = new Handler(Looper.getMainLooper())
	{
		@Override
		public void handleMessage(Message msg) 
		{
			//listener 判断是否为null
			if(mInitListener == null)
				return;
			mInitListener.onInit(ErrorCode.SUCCESS);
		}
	};
	
	
	protected void reInit(Context context) 
	{	
		//创建语记识别对象
		SpeechUtility utility = SpeechUtility.getUtility();
		if(null != utility && utility.isServiceInstalled()
				&& utility.getEngineMode() != ENGINE_MODE.MSC) 
		{
			//判断服务是否可用，在用户升级语记造成的服务端开时会出现服务不可用的情况
			if(null != mUnderstanderAidl && !mUnderstanderAidl.isAvailable())
			{
				mUnderstanderAidl.destory();
				mUnderstanderAidl = null;
			}
			mUnderstanderAidl = new SpeechUnderstanderAidl(context.getApplicationContext(), mInitListener);
		} 
		else if(null != mInitListener) 
		{
			if(null != mUnderstanderAidl)
			{
				mUnderstanderAidl.destory();
				mUnderstanderAidl = null;
			}
		}
	}

	/**
	 * <h4>开始语音语义</h4>
	 * <p>
	 * 		调用此函数，开始语音语义理解。
	 * </p>
	 * <p>
	 * 		目前SDK不支持多线程，所以在调用本函数开始一次会话后，直到结束前(结果返回
	 * 完毕，或出现错误)，不能再调用本函数开始新的会话。一次会话，即从会话开始，到结
	 * 束为止。
	 * </p>
	 * <p>
	 * 		在调用本函数开始会话前，通过{@link #setParameter(String, String)}设置相
	 * 应的参数。
	 * </p>
	 * <p>
	 * 		调用本函数开始语音语义后，通过SDK的录音机录取用户通过麦克风读入的音频（当
	 * {@link SpeechConstant#AUDIO_SOURCE} 值>=0时），或由应用层调用
	 * {@link #writeAudio(byte[], int, int)}写入音频流，获取用于语音语义理解的音频。在完成音频
	 * 录入（包括麦克风或写音频流方式）后，通过调用{@link #stopUnderstanding()}告知SDK已完
	 * 成音频录入，或由SDK自带的VAD（Voice Activity Detection,静音抑制）自动结束音频录
	 * 入，见{@link SpeechUnderstanderListener#onEndOfSpeech()}。（关于写入音流方式的SDK
	 * 自动结束录入应注意的地方，请参考{@link #writeAudio(byte[], int, int)}；VAD端点长度设置，请
	 * 参考{@link #setParameter(String, String)}）
	 * </p>
	 * <p>
	 * 		可通过{@link #cancel()}取消当前的会话。
	 * </p>
	 * 
	 * @see #createUnderstander
	 * @see #getUnderstander()
	 * @see #stopUnderstanding()
	 * @see #setParameter(String, String)
	 * @see #cancel()
	 */
	public int startUnderstanding(SpeechUnderstanderListener listener)
	{
		//语记已经安装，并且本地或者混合识别,需要走语记
		ENGINE_MODE mode = getStartMode(SpeechConstant.ENG_NLU, mUnderstanderAidl);
		DebugLog.LogD("start engine mode = " + mode.toString());
		//@{ start: 去除语记支持
		/*if(mode == ENGINE_MODE.PLUS)
		{
			//没有安装语记
			if(mUnderstanderAidl == null) 
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			mUnderstanderAidl.setParameter(SpeechConstant.PARAMS, null);
			mUnderstanderAidl.setParameter(SpeechConstant.PARAMS, mSessionParams.toString());
			
			//主线程抛出回调消息
			mInternalAidlListener = new InternalListener(listener);				
			return mUnderstanderAidl.startUnderstanding(mInternalAidlListener.mAidlListener);
		} 
		else 
		{
			if(mUnderstander == null)
			{
				return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
			}
			mUnderstander.setParameter(mSessionParams);
			return mUnderstander.startUnderstanding(listener);
		}*/
		//@ end:去除语记支持 }
		if(mUnderstander == null)
		{
			return ErrorCode.ERROR_COMPONENT_NOT_INSTALLED;
		}
		mUnderstander.setParameter(mSessionParams);
		return mUnderstander.startUnderstanding(listener);
	}

	/**
	 * <h4>是否在会话中</h4>
	 * <p>
	 * 		通过此函数，获取当前SDK是否正在进行会话。应用层可通过此函数，查询能否
	 * 开始一路新的会话等。
	 * </p>
	 * <p>
	 * 		调用了{@link #stopUnderstanding()}停止录音后，如果会话未出现错误或返回最后的结果，
	 * 当前状态依然处于会话中，即本函数会返回true。如果调用{@link #cancel()}取消了会话，
	 * 则当前状态处于不在会话中。
	 * </p>
	 * @return 会话状态，true：正在会话中；false：不在会话中。
	 * 
	 * @see #startUnderstanding(SpeechUnderstanderListener)
	 * @see #stopUnderstanding()
	 * @see #cancel()
	 */
	public boolean isUnderstanding() 
	{
		if(null != mUnderstander && mUnderstander.isUnderstanding()) 
		{
			return true;
		}
		else if(null != mUnderstanderAidl && mUnderstanderAidl.isUnderstanding()) 
		{
			return true;
		} 
		else 
		{
			return false;
		}
	}

	/**
	 * <h4>写入录音数据</h4>
	 * <p>
	 * 		通过调用此函数，把音频数据传给SDK。
	 * </p>
	 * <p>
	 * 		仅在{@link SpeechConstant#AUDIO_SOURCE}值为-1时，需要通过调用本函数，把
	 * 音频数据传给SDK。在写入音频数据前，应先通过
	 * {@link #startUnderstanding(SpeechUnderstanderListener)} 开启会话。在结束录音数据
	 * 写入后，调用{@link #stopUnderstanding()}告知应用层，当前会话音频数据已全部写入。
	 * </p>
	 * <p>
	 * 		当SDK回调{@link SpeechUnderstanderListener#onEndOfSpeech()}时，说明SDK已经
	 * 通过VAD检测到了静音末端点，应该立即停止写入音频数据。当SDK回调
	 * {@link SpeechUnderstanderListener#onResult(UnderstanderResult)}返回完结果，或回调
	 * {@link SpeechUnderstanderListener#onError(SpeechError)}返回错误时，也应该停止当次
	 * 会话的音频写入。
	 * </p>
	 * <p>
	 * 		支持的音频格式，请参考{@link #setParameter(String, String)}的参数说明。
	 * </p>
	 * 
	 * @param buffer
	 *            要写入的录音数据缓存
	 * @param offset
	 *            实际音频在传入缓存的起始点
	 * @param length
	 *            音频数据长度
	 * @return 
	 * 			  是否成功写入数据，{@link ErrorCode#SUCCESS}表示成功，其他值表示有错误，
	 * 具体错误码，请参考{@link ErrorCode}。
	 * 
	 * @see #startUnderstanding(SpeechUnderstanderListener)
	 * @see #stopUnderstanding()
	 * @see SpeechUnderstanderListener
	 */
	public int writeAudio(byte[] buffer, int offset, int length) {
		//TODO 需要测试写音频是否可用
		if(null != mUnderstander && mUnderstander.isUnderstanding())
		{
			return mUnderstander.writeAudio(buffer, offset, length);
		}
		else if(null != mUnderstanderAidl && mUnderstanderAidl.isUnderstanding())
		{
			//TODO 新增写音频接口
			return mUnderstanderAidl.writeAudio(buffer, offset, length);
		}
		else 
		{
			DebugLog.LogD("SpeechUnderstander writeAudio, is not understanding");
			return ErrorCode.ERROR_ENGINE_CALL_FAIL;
		}
	}

	/**
	 * <h4>停止录音</h4>
	 * <p>
	 * 		调用本函数告知SDK，当前会话音频已全部录入。
	 * </p>
	 * <p>
	 * 		在调用本函数后，已录入的音频还在继续上传到服务器，结果不会马上就返回，当前会
	 * 话还在继续，直到结果返回完毕，或出现错误。
	 * </p>
	 * <p>
	 * 		要取消会话，请参考{@link #cancel()}函数。
	 * </p>
	 * <p>
	 * 		当应用调用本函数结束停止录音时，SDK不会再回调
	 * {@link SpeechUnderstanderListener#onEndOfSpeech()}。相反的，当SDK回调
	 * {@link SpeechUnderstanderListener#onEndOfSpeech()}时，应用层可不必再调用本函数
	 * 通知SDK停止录音。
	 * </p>
	 * 
	 * @see #startUnderstanding(SpeechUnderstanderListener)
	 * @see #writeAudio(byte[], int, int)
	 * @see SpeechUnderstanderListener#onEndOfSpeech()
	 * @see #cancel()
	 */
	public void stopUnderstanding() 
	{
		if(null != mUnderstander && mUnderstander.isUnderstanding()) 
		{
			mUnderstander.stopUnderstanding();
		}
		else if(null != mUnderstanderAidl && mUnderstanderAidl.isUnderstanding())
		{
			mUnderstanderAidl.stopUnderstanding(mInternalAidlListener.mAidlListener);
		}
		else
		{
			DebugLog.LogD("SpeechUnderstander stopUnderstanding, is not understanding");
		}
	}
	
	/**
	 * <h4>取消会话</h4>
	 * <p>
	 * 		通过此函数取消当前的会话。
	 * </p>
	 * <p>
	 * 		在会话被取消后，当前会话结束，未返回的结果将不再返回。
	 * </p>
	 * 
	 * @see #startUnderstanding(SpeechUnderstanderListener)
	 * @see #stopUnderstanding()
	 */
	public void cancel() 
	{
		if(null != mUnderstander && mUnderstander.isUnderstanding()) 
		{
			mUnderstander.cancel(false);
		}
		else if(null != mUnderstanderAidl && mUnderstanderAidl.isUnderstanding())
		{
			mUnderstanderAidl.cancel(mInternalAidlListener.mAidlListener);
		}
		else
		{
			DebugLog.LogE("SpeechUnderstander cancel failed, is not running");
		}
	}
	
	/**
	 * <h4>参数设置</h4>
	 * <p>
	 * 		可设置的参数有：<br>
	 * {@link SpeechConstant#SCENE } ：情景模式 <br>
	 * {@link SpeechConstant#NLP_VERSION}：语义版本<br>
	 * {@link SpeechConstant#NET_TIMEOUT }： 网络连接超时时间<br>
	 * {@link SpeechConstant#KEY_SPEECH_TIMEOUT }：语音输入超时时间 <br>
	 * {@link SpeechConstant#LANGUAGE }：语言<br>
	 * {@link SpeechConstant#ACCENT }：语言区域<br>
	 * {@link SpeechConstant#DOMAIN }：应用领域<br>
	 * {@link SpeechConstant#AUDIO_SOURCE }：音频源<br>
	 * {@link SpeechConstant#BLUETOOTH}: 蓝牙录音<br>
	 * {@link SpeechConstant#VAD_BOS }：前端点超时<br>
	 * {@link SpeechConstant#VAD_EOS }：后端点超时<br>
	 * {@link SpeechConstant#VAD_ENABLE }：启用VAD<br>
	 * {@link SpeechConstant#SAMPLE_RATE }：识别采样率<br>
	 * {@link SpeechConstant#RESULT_TYPE }：识别结果类型<br>
	 * {@link SpeechConstant#ASR_AUDIO_PATH }：识别录音保存路径
	 * <br>{@link SpeechConstant#ENGINE_TYPE}：引擎类型（目前语义仅支持在线模式）；
	 * </p>
	 * <p>
	 * 		当前语音语义支持未压缩的16位，单声道，采样率为16000或8000，字节顺序
	 *  为Little-Endian的Windows PCM音频。
	 * </p>
	 * 
	 * @param key   参数名称
	 * @param value 参数值
	 * @return 是否设置成功
	 * 
	 * @see #getParameter(String)
	 * @see #startUnderstanding(SpeechUnderstanderListener)
	 */
	public boolean setParameter(String key, String value) {
		return super.setParameter(key, value);
	}
	
	/**
	 * <h4>获取参数</h4>
	 * <p>
	 * 		获取指定的参数的当前值。
	 * </p>
	 * <p>
	 * 		某些有默认的值的参数，如{@link SpeechConstant#SAMPLE_RATE}，在应用层未
	 * 调用{@link #setParameter(String, String)}来设置参数值前，可能获取到的是null值，
	 * 但在实际会话会始时，SDK会使用默认值传给服务器。
	 * </p>
	 * 
	 * @param key
	 *            参数名称，所有支持参数，参考{@link #setParameter(String, String)}
	 * @return 参数值，值意义，参考{@link SpeechConstant}对各参数的说明。
	 * 
	 * @see #setParameter(String, String)
	 */
	public String getParameter(String key) {
		return super.getParameter(key);
	}

	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createUnderstander}创建的单例对象。
	 * </p>
	 * <p>
	 * 		在调用本函数进行销毁前，应先保证当前不在会话中，否则，本函数将尝试取消当前
	 * 会话，并返回false，此时销毁失败。关于当前是否在会话中，请参考函数
	 * {@link #isUnderstanding()}。若销毁失败，请在取消当前会话后，再次调用本函数重试。
	 * </p>
	 * <p>
	 * 		当本函数返回true时，销毁成功。此时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createUnderstander}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @return 销毁成功：true；销毁失败：false。 
	 * @see #createUnderstander
	 * @see #cancel()
	 */
	public boolean destroy() {
		boolean ret = true;
		final SpeechUnderstanderAidl  aidl = this.mUnderstanderAidl;
		if( aidl != null ) {
			aidl.destory();
		}
		
		synchronized( this ){
			mUnderstanderAidl = null;
		}//end of sync
		
		final SpeechUnderstanderImpl impl = this.mUnderstander;
		if( impl != null ){
			ret = impl.destroy();
		}
		
		if( ret && (ret=super.destroy()) ){
			synchronized( sSync ){
				sInstance = null;
			}//end of sync
		}
		
		return ret;
	}
	
	
	
	
	
	
	/**
	 * 内部语义理解监听类，保证回调在主线程完成
	 * 
	 *
	 */
	private final class InternalListener implements SpeechUnderstanderListener
	{
		private SpeechUnderstanderListener mOutListener = null;
		private com.iflytek.speech.SpeechUnderstanderListener mAidlListener = null;
		
		
		private static final int MSG_ERROR = 0;
		private static final int MSG_AUDIO_BUFFER = 1;
		private static final int MSG_SPEECH_BEGIN = 2;
		private static final int MSG_SPEECH_END = 3;
		private static final int MSG_RESULT = 4;
		// EVENT自定义消息
		private static final int MSG_EVENT  = 6;
		
		public InternalListener(SpeechUnderstanderListener outListener)
		{
			mOutListener = outListener;
			mAidlListener = new com.iflytek.speech.SpeechUnderstanderListener.Stub() {
				
				@Override
				public void onVolumeChanged(int volume, byte[] data) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onResult(com.iflytek.speech.UnderstanderResult result) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_RESULT, new UnderstanderResult(result.getResultString()));
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onError(int errorCode) throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_ERROR, new SpeechError(errorCode));
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onEndOfSpeech() throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END);
					mUiHandler.sendMessage(msg);
				}
				
				@Override
				public void onBeginOfSpeech() throws RemoteException {
					Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN);
					mUiHandler.sendMessage(msg);					
				}

				@Override
				public void onEvent(int eventType, int arg1, int arg2,
						Bundle obj) throws RemoteException {
					Message msgbody = new Message();
					msgbody.what = eventType;
					msgbody.arg1 = arg1;
					msgbody.arg2 = arg2;
					msgbody.obj  = obj;
					
					Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0,0, msgbody);
					mUiHandler.sendMessage(tmsg);
				}
			};
		}
		
		private Handler mUiHandler = new Handler(Looper.getMainLooper()) {
			@Override
			public void handleMessage(Message msg) {
				if (mOutListener == null)
					return;
				switch (msg.what) {
				case MSG_ERROR:
					mOutListener.onError((SpeechError) msg.obj);
					break;
				case MSG_AUDIO_BUFFER:
					mOutListener.onVolumeChanged(msg.arg1, (byte[])msg.obj);
					break;
				case MSG_SPEECH_BEGIN:
					mOutListener.onBeginOfSpeech();
					break;
				case MSG_SPEECH_END:
					mOutListener.onEndOfSpeech();
					break;
				case MSG_RESULT:
				{
					mOutListener.onResult((UnderstanderResult)msg.obj);
				}
					break;
				case MSG_EVENT:
					Message mbody = (Message)msg.obj;
					// 自定义消息数据
					mOutListener.onEvent(mbody.what,mbody.arg1,mbody.arg2,(Bundle)mbody.obj);
					break;
				default:
					break;
				}
				super.handleMessage(msg);
			}
		};
		
		@Override
		public void onVolumeChanged(int volume, byte[] data) {
			Message msg = mUiHandler.obtainMessage(MSG_AUDIO_BUFFER, volume, 0,data);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEndOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_END, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onResult(UnderstanderResult result) {
			Message msg = mUiHandler.obtainMessage(MSG_RESULT, result);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onBeginOfSpeech() {
			Message msg = mUiHandler.obtainMessage(MSG_SPEECH_BEGIN, 0, 0, null);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onError(SpeechError error) {
			Message msg = mUiHandler.obtainMessage(MSG_ERROR, error);
			mUiHandler.sendMessage(msg);
		}

		@Override
		public void onEvent(int eventType, int arg1, int arg2, Bundle obj) {
			
			Message msgbody = new Message();
			msgbody.what = eventType;
			msgbody.arg1 = arg1;
			msgbody.arg2 = arg2;
			msgbody.obj  = obj;
			
			Message tmsg = mUiHandler.obtainMessage(MSG_EVENT, 0,0, msgbody);
			mUiHandler.sendMessage(tmsg);
		}
		
	}
}