package com.iflytek.cloud;

import android.os.Bundle;


/**
 * <h3>语音语义监听器</h3>
 * <p>通过实现此接口，获取当前语音语义理解的状态和结果</p>
 * <p>合成监听器，请参考{@link SynthesizerListener}</p>
 * <p>识别监听器，请参考{@link SpeechRecognizer}</p>
 * <p>文本语义监听器，请参考{@link TextUnderstanderListener}</p>
 *
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see SpeechUnderstander#startUnderstanding(SpeechUnderstanderListener)
 * @see SynthesizerListener
 * @see SpeechRecognizer
 * @see TextUnderstanderListener
 */
public interface SpeechUnderstanderListener {
	
	/**
	 * <h4>音量变化</h4>
	 * <p>
	 *     当会话开始，到停止录音（停止写入音频流）或SDK返回最后一个结果自动结束识别为止，
	 * SDK检测到音频数据（正在录音或写入音频流）的音量变化时，会多次通过此函数回调，
	 * 告知应用层当前的音量值。应用层可通过此函数传入的值变化，改变自定义UI的画面等。
	 * </p> 
	 * @param volume 当前音量值，范围[0-30]
	 * @see SpeechUnderstander#startUnderstanding(SpeechUnderstanderListener)
	 * @see #onBeginOfSpeech
	 * @see #onEndOfSpeech
	 * @see SpeechUnderstander#stopUnderstanding()
	 */
	void onVolumeChanged(int volume, byte[] data);
	
	/**
	 * <h4>开始说话</h4>
	 * <p>
	 * 		在录音模式({@link SpeechConstant#AUDIO_SOURCE 音频源}参数设为 > -1时 )下，
	 * 调用{@link SpeechUnderstander#startUnderstanding(SpeechUnderstanderListener)
	 *  开始录音函数}后，会自动开启系统的录音机，并在录音机开启后，会回调此函数（这中间
	 *  的过程应该在几毫秒内，可以忽略，除非系统响应很慢）。
	 * </p>
	 * <p>
	 * 		应用层可通过此函数回调，告知用户，当前可开始说话。若应用层使用的是声音提示，
	 * 则应该在调用 {@link SpeechUnderstander#startUnderstanding 开始录音函数}前，提放
	 * 提示音，并在提示音非静音数据播放结束时，调用
	 * {@link SpeechUnderstander#startUnderstanding 开始录音函数}，开始录音。若太早调
	 * 用，则可能会把提示音录进要识别的音频中，若太晚，则可能会漏掉部分用户说话的音频。
	 * </p>
	 * @see SpeechUnderstander#startUnderstanding
	 * @see #onEndOfSpeech
	 */
	void onBeginOfSpeech();

	/**
	 * <h4>结束说话</h4>
	 * <p>
	 * 		在SDK检测到音频的静音端点时，回调此函数（在录音模式或写音频模式下都会回调，
	 * 应用层主动调用{@link SpeechUnderstander#stopUnderstanding()}则不会回调此函数，
	 * 在识别出错时，可能不会回调此函数）。
	 * 在此函数回调后，当前会话可能并没有结束，结果可能还要等待一定的时间才会返回。
	 * </p>
	 * <p>
	 * 		此函数回调后，应用层应立即停止调用{@link SpeechUnderstander#writeAudio}写入
	 * 音频数据，(当音频源设置为音频流，{@link SpeechConstant#AUDIO_SOURCE}为-1时）
	 * 否则，再通过{@link SpeechUnderstander#writeAudio}写入的音频也会被忽略。
	 * </p>
	 * <p>
	 * 		应用层可以通过此函数回调，告知用户，当次说话已结束，正在等待识别结果（若结果
	 * 未返回）等。
	 * </p>
	 * 
	 * @see #onBeginOfSpeech
	 * @see #onResult
	 * @see #onError
	 * @see SpeechUnderstander#writeAudio
	 * @see SpeechUnderstander#stopUnderstanding()
	 */
	void onEndOfSpeech();

	/**
	 * <h4>返回结果</h4>
	 * <p>
	 * 		返回的结果可能为null，请增加判断处理。
	 * </p>
	 * <p>
	 * 		一次会话中的语义理解结果会一次性返回。当结果返回时，本次会话结束，录
	 * 音也会停止，在重新调用 
	 * {@link SpeechUnderstander#startUnderstanding(SpeechUnderstanderListener)}开启新
	 * 的识别会话前，停止调用{@link SpeechUnderstander#writeAudio}写入音频(当音频源设置为
	 * 音频流，{@link SpeechConstant#AUDIO_SOURCE}为-1时）。当出现错误，或应用层调用
	 * {@link SpeechUnderstander#cancel}取消当次会话时，在当次会话过程可能不会回调此函数。
	 * </p>
	 * 
	 * @param result 结果数据
	 * 
	 * @see #onError
	 * @see SpeechUnderstander#cancel
	 * @see UnderstanderResult
	 */
	void onResult(UnderstanderResult result);

	/**
	 * <h4>错误回调</h4>
	 * <p>
	 * 		当此函数回调时，说明当次会话出现错误，会话自动结束，录音也会停止。应在再次调用
	 * {@link SpeechUnderstander#startUnderstanding(SpeechUnderstanderListener)}开启新
	 * 的会话前，停止调用{@link SpeechUnderstander#writeAudio}写入音频(当音频源设置为
	 * 音频流，{@link SpeechConstant#AUDIO_SOURCE}为-1时）。
	 * </p>
	 * @param error 错误信息
	 * 
	 * @see SpeechError
	 * @see #onResult
	 * @see SpeechUnderstander#startUnderstanding
	 */
	void onError(SpeechError error);
	
    /**
     * <h4>事件回调</h4>
     * <p>
     * 		扩展用接口，由具体业务进行约定。例如eventType为0显示网络状态，agr1为网络连接值。
     * </p>
     * @param eventType 消息类型
     * @param arg1 参数1
     * @param arg2 参数2
     * @param obj 消息内容
     * 
     * @see SpeechEvent
     */
    void onEvent(int eventType, int arg1,int arg2,Bundle obj);
}
