
package com.iflytek.cloud.util;

import com.iflytek.cloud.util.contacts.ContactManagerImpl;

import android.content.Context;


/**
 * <h3>联系人管理类</h3>
 * <p>
 * 		联系人管理类，用来获取联系人姓名等数据，用于听写词典等。
 * </p>
 * <p>
 * 		联系人管理类仅作为辅助工具查询联系人姓名，用于听写时的词典上传到服务器以提高
 * 听写时对联系人的匹配（见{@link com.iflytek.cloud.SpeechRecognizer#updateLexicon}），
 * 不会上传联系人的电话号码，请放心使用。您也可以自己实现对联系人名字获取，再通过
 * {@link com.iflytek.cloud.SpeechRecognizer#updateLexicon}上传更新即可。
 * 通过些功能获取系人时，应用需拥有以下权限：
 * <br>{@link android.Manifest.permission#READ_CONTACTS}
 * </p>
 * <p>
 * 		结果格式如下：
 * <br>张三<br>李四<br>王五
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createManager}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getManager}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createManager}再次创建单例对象，方可再使用。
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see com.iflytek.cloud.SpeechRecognizer#updateLexicon
 */

public abstract class ContactManager {

/**
     * <h4>获取实例</h4>
     * <p>
     * 		获取已创建的实例，若返回null，则应该先通过{@link #createManager}创建实例。
     * </p>
     * 
     * @see #createManager
     * @return ContactManager的实例
     */

    public static ContactManager getManager() {
        return ContactManagerImpl.getManager();
    }
    

/**
     * <h4>联系人查询回调</h4>
     * <p>
     * 		通过实现此接口，并在{@link ContactManager#createManager}传入，通过
     * {@link ContactManager#asyncQueryAllContactsName}开始异步查询后，通过
     * 此接口获取联系人查询状态和结果。
     * </p>
     * 
     * @see ContactManager#createManager
     */

    public interface ContactListener {

/**
    	 * <h4>联系人查询结束</h4>
    	 * <p>
    	 * 		通过此接口，获取查询结果。在调用{@link ContactManager#asyncQueryAllContactsName}
    	 * 开始查询后，将通过此函数返回一次结果。如果随后联系人或通话记录有变化，此函数将再次回调，
    	 * 并通过参数2告知是否是联系或通话记录有变化。
    	 * </p>
    	 * @param contactNames 联系人姓名集字符串，格式说明，请参考{@link ContactManager}类说明。
    	 * @param changeFlag 联系人是否变化标志
    	 * 
    	 * @see ContactManager#createManager
    	 * @see ContactManager#asyncQueryAllContactsName
    	 * @see ContactManager#queryAllContactsName
    	 */

        void onContactQueryFinish(String contactInfos, boolean changeFlag);
    }
    

/**
     * <h4>创建实例</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getManager}获取已创建的单例对象。
	 * </p>
     * 
     * @param context	应用上下文
     * @param contactListener 查询联系人监听器，用以获取查询结果
     * @return ContactManager对象
     * 
     * @see #getManager
     * @see #destroy
     */

    public static ContactManager createManager(Context context, ContactListener contactListener) {
        return ContactManagerImpl.createManager(context, contactListener);
    }
    

/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，停止查询联系人和监听联系人变化，并销毁由{@link #createManager}
	 * 创建的单例对象。
	 * </p>
	 * <p>
	 * 		销毁成功时，之前创建的单例对象已不能再使用，否则，
	 * 将会报错。此时需要再使用，应先通过{@link #createManager}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @see #createManager
	 */

    public static void destroy(){
    	ContactManagerImpl.destroy();
    }
    

/**
     * 初始化 ContactManager对象。<br>
     */

    protected ContactManager() {
    	
    }
    

/**
     * <h4>同步查询联系人名</h4>
     * <p>
     * 		同步查询，直接通过函数返回联系人名，会有一定的线程阻塞时间。
     * </p>
     * @return 联系人名称，格式说明请参考本类说明
     * 
     * @see #asyncQueryAllContactsName
     * @see #getManager
     */

    public abstract String queryAllContactsName();
    

/**
     * <h4>异步查询联系人名</h4>
     * <p>
     * 		异步查询，通过{@link ContactManager.ContactListener}返回查询结果，不会阻塞调用线程。
     * 查询状态和结果监听器{@link ContactManager.ContactListener}设置，参考
     * {@link #createManager}。
     * </p>
     * 
     * @see #queryAllContactsName
     * @see #getManager
     * @see ContactManager.ContactListener
     */

    public abstract void asyncQueryAllContactsName();

}

